#! /usr/bin/env python
#
# @file
# OTF mosaic patterns, taken from continuousIntegMosaic.py
# these are the new style mosaics implemented April 2012
#
# Note that for centered grids, the number of rows (NR) needs to
# be odd, and the number of columns needs to be even/odd 
# depending if (NR+1)/2 is even/odd, if you want (0,0) to be
# included in your pointings. A warning is printed in the output
# if your grid selection does not include the center.

# @author Steve Scott (RTS), Peter Teuben (MIS)
# $Id: OTFmosaic.py,v 1.4 2012/05/16 14:08:10 pteuben Exp $
#
# $CarmaCopyright$

#   

if True:
    # these are to fake using it offline
    DEFAULT = 0
    import sys, math, parfile
else:
    # these are for the CARMA RTS 
    from subarrayCommands import *


class Mosaic(object):
    def __init__(self, fieldtime, rows, fieldsInRow, delta=0.5, pa=0.0, 
            maxtime=20.0, maxreps=0, startFieldIndex=0, trkThreshold=0.07,
            slewtime=1.5, rowOffset=0.0, colOffset=0.0):
        """ The mosaic is a quasi-rectangle with rows of equally spaced fields
        centered on the current source position.
        The fields are ordered from the upper left, then moving along a row
        from left to right, one field to another. The next row is done left to
        right, with the pattern repeating to give an efficient serpentine 
        traversal of the fields. The rows are spaced by root 2 of the field
        spacing and even rows have one additional field and are offset 
        by half a fields to make all fields equally spaced from any adjacent
        field. By default the mosaic is repeated ad infinitum, with even 
        repetitions in the reverse direction from the odd. This gives fields
        with no large discontinuities.
        The code internally uses the equatOffset command and moves the 
        phase center as well as the pointing center.
        Parameters:
         fieldtime: integration time on each field (secs). The actual data
             integration time will be the sum of the slewtime and fieldtime.
         rows: number of rows of fields
         fieldsInRow: number of fields in odd numbered rows (even have one more)
         delta: field spacing in arcminutes. Default is 0.5.
         pa: position angle of the rectangle in degrees. The default, 0. gives
             a rectangle with rows in RA. Increasing pa rotates the rectangle
             around the nominal source position from north toward east.
        maxtime: the maximum time in minutes to take data. This allows 
             calibrations to be scheduled at fixed intervals.
        maxreps: the maximum number of repeats of the mosaic. If both maxreps
             and maxtime are specified the first one to complete wins.
        startFieldIndex: index of first field, defaults to 0
        trkThreshold: tracking threshold in fractions of a beamwidth,
             default is 0.07. 
        slewtime: estimated time between fields for the antennas to move.
             The data at the start of the integration during the slew should
             be blanked because of tracking.     
        rowOffset: An offset in arcminutes to add to the center y position,
                   allowing a non-centered rectangle.  Default: 0
        colOffset: An offset in arcminutes to add to the center x position,
                   allowing a non-centered rectangle.  Default: 0
        """
        self.fieldtime     = fieldtime
        self.nrows         = rows
        self.ncols         = fieldsInRow
        self.delta         = delta
        self.pa            = pa
        self.maxtime       = maxtime
        self.slewtime      = slewtime
        self.maxreps       = maxreps
        self.trkThreshold  = trkThreshold
        self.fieldIndex    = startFieldIndex
        self.rowOffset     = rowOffset
        self.colOffset     = colOffset
        self._makePositions(rows, fieldsInRow, delta, pa)
        self.nFields       = len(self.mosaicFields)
        self.maxFieldIndex = maxreps*self.nFields
        m = "# Creating a %dx%d mosaic with %d positions at pa=%.1f deg" \
                %(rows, fieldsInRow, self.nFields, pa)
        print m
        #scriptlog(m)
    def _makePositions(self, nrows, ncols, delta, pa) :    
        self.mosaicFields = list()
        for r in range(nrows) :
            firstRow = (r==0)
            odd  = (r & 1) == 1
            even = (r & 1) == 0
            nRAfields = ncols
            if odd : nRAfields = ncols+ 1
            deco = self.rowOffset + delta*math.sqrt(3.0)/2*(0.5*(nrows-1)-r)
            for d in range(nRAfields):
                if even: rao = self.colOffset+delta*(d-0.5*(nRAfields-1))
                else   : rao = self.colOffset+delta*(0.5*(nRAfields-1)-d)
                # Now rotate by pa
                r = math.hypot(rao, deco)
                a = math.atan2(deco, rao) - pa*math.pi/180
                decopa = r*math.sin(a)
                raopa  = r*math.cos(a)
                self.mosaicFields.append([raopa, decopa])

        #print self.dumpFields()
    def dumpFields(self):
        """Return a string with each field position on a separate line as
        ra, dec offset pairs."""
        o = ""
        i = 1
        for f in self.mosaicFields:
            o += "%3d: %6.2f, %6.2f\n" %(i, f[0],f[1])
            i += 1
        return o
    def reset(self):
        """Reset the field index to zero to restart the mosaic from scratch"""
        self.fieldIndex = 0
    def _getField(self, completionMsg=False, loud=True):
        """Return ra/dec offset pair for current field. Also useful to print
        out info about mosaic start/stop."""
        maprep = (self.fieldIndex)/self.nFields
        fieldInThisRep = (self.fieldIndex)%self.nFields
        even = (maprep & 1) == 0
        if even: index = fieldInThisRep
        else :   index = self.nFields-fieldInThisRep-1
        if even: dir = "forward"
        else:    dir = "reverse"
        bar  = "=========================================="
        m = ""
        if completionMsg:
            if fieldInThisRep == (self.nFields-1):
                m = "Completing mosaic rep %d\n" %(maprep+1)
                m += bar
        else:
            if self.fieldIndex == 0:
                m  = bar + "\n"
                m += "Starting first rep (forward direction)"
            elif fieldInThisRep == 0 :
                m = bar + "\n"
                m += "Starting mosaic rep %d (%s direction)" %(maprep+1,dir)
        if loud and len(m) > 0:
            print m
            scriptlog(m)        
        return self.mosaicFields[index]
        
    def _takeData(self, maxtime, waitTracking, subarray, fastSample) :
        """ Take data until maxtime or maxreps is exceeded. Will not 
        necessarily stop on a mosaic boundary. Starts from current field
        position, which is either the start of the mosaic is the first 
        takeData, or where the previous takeData finished. This allows 
        data collection to be started/stopped/restarted for calibrations.
        fastSample = False # When true, does 0.5sec integrations"""
        hasPipeline = True   # Set to False for sandbox testing
        dbgtt       = False  # Control track threshold debug prints
        if maxtime == None: maxtime = self.maxtime
        if (maxtime <= 0) and (self.maxreps <= 0) :
            m = "Both maxtime and maxMosaics cannot be unlimited "
            m += "which is done by setting them to zero."
            raise Exception, m
        looptime      = 0.05
        blankDuration = 1.0  # Length of blanking (secs) when doing field slew
        blankOffset   = 0.0
        fieldtime     = self.fieldtime
        slewtime      = self.slewtime
        iterationTime = fieldtime+slewtime 
        # Integration time
        if fastSample : itime         = 0.5     
        else          : itime         = iterationTime     
        tbegin    = time.time()
        trackThreshold(self.trkThreshold)
        
        src = queryString("Control.Subarray"+str(subarrayNo)+".source")
        #intent(src, 'S',True,False)
        
        f0   = self._getField()
        rao  = f0[0]
        deco = f0[1]
        equatOffset(rao, deco)
        if waitTracking:
            c1 = "Waiting for most antennas to acquire source"
            c2 = "Most antennas acquired"
            tmo = 500 # In seconds, to allow for a long slew
            wait(TRACK, tmo=tmo, waiton=-2, precomment=c1, postcomment=c2)
            c1 = "Waiting for last two antennas to acquire source"
            c2 = "Last two antennas acquired"
            tmo = 4 # Give stragglers a second chance
            wait(TRACK, tmo=tmo, waiton=0, precomment=c1, postcomment=c2)
        else:
            wait(TIME, tmo=slewtime, subarray=subarray)
        
        nFields    = self.nFields
        fieldsLeft = nFields - self.fieldIndex
        telapsed   = time.time()-tbegin
        if maxtime*60 > iterationTime+telapsed:
            mf        = (maxtime*60.0-telapsed)/iterationTime
            maxtimefields = max(int(round(mf)), 0)
            if self.maxFieldIndex > 0:
                fieldsLeft = self.maxFieldIndex-self.fieldIndex
                fieldsLeft = min(fieldsLeft, maxtimefields)
            else:    
                fieldsLeft = maxtimefields
        if fieldsLeft <= 0:
            print "No fields left to do, exiting"
            return
        reps = int(round((fieldsLeft*iterationTime)/itime))
        rtdComment("Integrating...") 
        multiSubarray('integrate', subarray, itime, reps)
        mp = "SlPipeline.IntegratorStageContainer.IntegratorStage.integrating"
        if hasPipeline:
            while (queryBool(mp)):
                sleep(0.1)
        else : 
            sleep(1.5)
        tintegstart = time.time()
        olddeco = deco
        field     = 0
        prefield  = 0
        postfield = 0
        print "%5.1f" %0.0, "%2d" %field, "%5.2f" %rao, "%5.2f" %deco
        t0 = time.time() + 1.30 # The last offset is from pipeline
        hasSlew = False
        while(postfield < (fieldsLeft-1)) :
            hasSlew = True
            now  = time.time()
            t = now - t0
            nextField     = int(t/iterationTime)
            nextPrefield  = int((t+blankOffset)/iterationTime)
            nextPostfield = int((t+blankOffset-blankDuration)/iterationTime)
            #print "%.1f" %t, field, nextPrefield, nextField, nextPostfield
            if (nextPrefield > prefield) :
                prefield = nextPrefield
                trackThreshold(0.0)
                if dbgtt: print "%5.1f" %t, "Start blanking everything" 
            if (nextField > field) :
                field = nextField
                self._getField(completionMsg=True)
                self.fieldIndex += 1
                f = self._getField()
                rao  = f[0]
                deco = f[1]
                equatOffset(rao, deco, whileIntegrating=True)
                tslewstart = time.time()
                firstFieldInRep = (self.fieldIndex%self.nFields) == 0
                if abs(olddeco-deco) < 0.0001 and not firstFieldInRep: 
                    decostr = ""
                else: 
                    decostr = "%5.2f" %deco
                olddeco = deco
                print "%5.1f" %t, "%2d" %field, "%5.2f" %rao, decostr
            if (nextPostfield > postfield) :
                postfield = nextPostfield
                trackThreshold(self.trkThreshold)
                if dbgtt: print "%5.1f" %t, "Normal blanking" 
            sleep(looptime)
        if hasPipeline:
            pc ="Integration complete"
            rtn = wait(INTEG, postcomment=pc, subarray=subarray)
        else : 
            rtn = wait(TIME, tmo=fieldtime, subarray=subarray)
        self._getField(completionMsg=True)
        self.fieldIndex += 1
        if hasSlew: lastField = time.time() - tslewstart
        else:       lastField = time.time() - tintegstart
        print "Length last field integration: %.2f secs" %lastField
        print "Total time: %.1fsecs" %(time.time()-tbegin)
        return self.fieldIndex

    def takeData(self, maxtime=None, waitTracking=True,
            subarray=DEFAULT, fastSample=False) :
        """ Take data until maxtime or maxreps is exceeded. Will not 
        necessarily stop on a mosaic boundary. Starts from current field
        position, which is either the start of the mosaic is the first 
        takeData, or where the previous takeData finished. This allows 
        data collection to be started/stopped/restarted for calibrations.
        Parameters:
         fastSample: when True takes data every frame for debugging. 
            Default is False.
        """
        # The whole purpose of this function to make sure the track threshold
        # is reset after an exception of any kind.
        try:
            self._takeData(maxtime, waitTracking, subarray, fastSample)
        except Exception, ex:
            trackThreshold(self.trkThreshold)
            raise Exception, ex
            
        
if __name__ == '__main__':
    def has_center(rows,cols):
        def is_even(n):
            if n%2 == 0: return True
            return False
        if is_even(rows): return False
        if is_even(cols):
            if is_even((rows+1)/2): return True
            return False
        else:
            if is_even((rows+1)/2): return False
            return True
        # should never get here
        print "has_center: Should never get here"
        return False
    p = parfile.ParFile()
    p.set('fieldtime',10.0)
    p.set('rows',5)
    p.set('cols',4)
    p.set('delta',0.5)
    p.set('pa',0.0)
    p.set('row0',0.0)
    p.set('col0',0.0)
    p.argv(sys.argv)

    fieldtime   = p.getf('fieldtime')
    rows        = p.geti('rows')
    cols        = p.geti('cols')
    delta       = p.getf('delta')
    pa          = p.getf('pa')
    row0        = p.getf('row0')
    col0        = p.getf('col0')

    m = Mosaic(fieldtime,rows,cols,delta,pa,rowOffset=row0,colOffset=col0)
    print m.dumpFields()
    print "# rows=%d cols=%d delta=%f pa=%f row0=%f col0=%f" % (rows,cols,delta,pa,row0,col0)
    if has_center(rows,cols):
        print "# Note: center is included"
    else:
        print "# Warning: center is not included in grid"
