#!/bin/tcsh -f
#
# Script to combine Singledish and CARMA data with "Linear Combination"
# method of Stanimirovic et al. 1999 MNRAS
#
# 99oct19 tth / modified 06nov99 tw
# Adapted for generic single dish and cleaned up csh 
#    - mwp Tue Apr 11 16:11:34 EDT 2000
# Modifications for CARMA and to use custom 'get' scripts -  mwp July 2009
# 
# $Id: lincomb.csh,v 1.10 2012/03/26 20:20:06 mpound Exp $

set path = ( $MIS/bin $MIRBIN $path )

set B="\033[1m"   # bold
set N="\033[0m"  # normal
set myname=`basename $0` 

onintr Intrupt

# dirty map, dirty beam, single dish map 
unset intmap  # dirty map
unset intbeam # dirty beam
unset intcm   # restored map
# Input singledish map, typically in temperature units.
# This script will regrid and convert to Janskys using jyperk keyword
unset sdmap
unset sdbmsz
unset jyperk
unset chan

# default display parameters
set device=/null
set nxy=4,4

# default MOSSDI iterations
set niters=100
# Sensitivity level in Jy/beam above which to mask the image.
# This keeps mossdi from finding components near map edges
set mask=0.5

# Region over which to measure flux and rms
# NOTE: These are only good for HCN test map!!!
set emreg="arcsec,box(-250,-400,200,400)"
# interferometric
set intrmsreg="image(1,5)"
# singledish
#set sdrmsreg="image(4,5)"
set sdrmsreg='arcsec,box(-350,-450,0,-300)(1)'
# for subimage

# pipeline can override them, and grab old variable
pipepar -f lincomb.def -c -s csh > tmp$$.par;  source tmp$$.par;  rm tmp$$.par

# command line overrides (just a cute CSH trick)
foreach _arg ($*)
  if ( $_arg == "-h" ) goto Help
  set $_arg
end

if ($?intmap == 0 ) then
  echo intmap must be given 
  goto Usage
endif
if ($?intbeam == 0 ) then
  echo intbeam must be given 
  goto Usage
endif
if ($?intcm == 0 ) then
  echo intcm must be given 
  goto Usage
endif
if ($?sdmap == 0 ) then
  echo sdmap must be given 
  goto Usage
endif
if ($?jyperk == 0 ) then
  echo jyperk must be given 
  goto Usage
endif

if ($?sdbmsz == 0 ) then
  echo Singledish beamsize will be computed from the data
endif


# Mossdi cleaning cutoff - set to 2x the rms in dirty map
set intrms=`getrms in=$intmap region=$intrmsreg`
if ( $?cutoff == 0 ) then 
    set cutoff=`calc "2.0*$intrms"`
endif
set cleangain=0.3
#set scale=1.0  # calibration scale factor

#goto Restore
# --------------------------------------------------------------------------
# 1. Prepare files for combination.
# --------------------------------------------------------------------------

# Compute the sdbmsz from the telescope and 
# frequency if it was not given on the command line.
if ( $?sdbmsz == 0 ) then 
  set telescop=`itemize in=$intmap/telescop | tail -1 |awk '{print $3}'`
  set restfreq=`itemize in=$intmap/restfreq | tail -1 |awk '{print $3}'`
  set sdbmsz=`pbplot telescop=$telescop freq=$restfreq | grep FWHM| awk '{print $3*60.0}'`
  echo "Computed beamsize is $sdbmsz arcsec"
else 
  echo "Using beamsize $sdbmsz arcsec"
endif

set origintmap=$intmap
set origintcm=$intcm
if ( $?chan != 0 ) then
  echo "Grabbing channels $chan from $intmap and $intcm"
  rm -rf $intmap.sub $intcm.sub
  imsub in=$intmap out=$intmap.sub region=quart'('$chan')'
  imsub in=$intcm out=$intcm.sub region=quart'('$chan')'
  set intmap=$intmap.sub
  set intcm=$intcm.sub
  minmax in=$intcm
  minmax in=$intmap
endif

#--------------------------------------------
# Set output names after final intmap set.
#
# Output of mospsf -- used in restore step
set psf=$intmap.psf
# output of mossen
set gain=$intmap.gain
set sen=$intmap.sen

# singledish beam file
set sdbeam=$intmap.sdbeam
# linear combination output
set of=$intmap.lcomb
# difference map
set diff=$of.diff
#--------------------------------------------

# Use MIRIAD regrid and maths tasks to regrid input singledish map
# Add some header variables to the singledish map
set regriddedmap=$sdmap.regrid
/bin/rm -rf $regriddedmap $sdmap.jy
set naxis=`itemize in=$intmap/naxis|tail -1 |awk '{print $3}'`
# make fake 4th axis if necessary
if ( $naxis == 3 ) then
   bigger $sdmap
endif
# This is wrong if they actually have different crval3 and crpix3!!!
#set crval3=`itemize in=$intmap/crval3|tail -1 |awk '{print $3}'`
#set crpix3=`itemize in=$intmap/crpix3|tail -1 |awk '{print $3}'`
#puthd in=$sdmap/crval3 value=$crval3 type=d
#puthd in=$sdmap/crpix3 value=$crpix3 type=d
regrid tin=$intmap in=$sdmap axes=1,2,3 out=$regriddedmap
maths exp='<'$regriddedmap'>*'$jyperk out=$sdmap.jy
puthd in=$sdmap.jy/bmaj value=$sdbmsz,arcsec type=d
puthd in=$sdmap.jy/bmin value=$sdbmsz,arcsec type=d
puthd in=$sdmap.jy/bpa  value=0 type=d
minmax in=$sdmap.jy

#
# Generate singledish beam and truncate it at 5% level.
#
echo "Creating SD beam and truncating at 5% level"
\rm -rf $sdbeam
imgen in=$intbeam factor=0 out=$sdbeam spar=1,0,0,$sdbmsz,$sdbmsz,0
\rm -rf $sdbeam.trunc
maths exp='<'$sdbeam'>' mask='<'$sdbeam'>'.ge.0.05 out=$sdbeam.trunc
minmax in=$sdbeam.trunc
#mv $sdbeam.trunc/mask $sdbeam.trunc/maskold

#
# Weight Single dish map by CARMA gain
#
/bin/rm -rf $gain $sen
mossen in=$intmap gain=$gain sen=$sen
echo "Applying CARMA gain to Singledish map..."
#
# move the mask so that maths works.
#
#if (-e $sdmap.jy/mask) mv $sdmap.jy/mask $sdmap.jy/maskold
#if (-e $gain/mask) mv $gain/mask $gain/maskold

#goto Convolve

ShowSize:
echo '$sdmap.jy size:'
itemize in=$sdmap.jy/naxis1
itemize in=$sdmap.jy/naxis2
itemize in=$sdmap/naxis3
echo 'gain size:'

itemize in=$gain/naxis2
itemize in=$gain/naxis3

Convolve:
echo "Convolving CARMA data to Singledish resolution..."
\rm -rf $sdmap.gain
echo maths exp="<$sdmap.jy>*<$gain>" out=$sdmap.gain
maths exp="<$sdmap.jy>*<$gain>" out=$sdmap.gain
\rm -rf $intcm.$sdbmsz
convol map=$intcm fwhm=$sdbmsz pa=0 options=final out=$intcm.$sdbmsz

#
# Calculate fluxes in each cube
#
set delv=`gethd in=$intcm/cdelt3`
set bflux=`getflux in=$intcm region=$emreg`
# Parenthesis needed because $delv can be less than zero
# which would cause calc to barf with:
# ### Fatal Error [(NOT SET)]:  Syntax error in the arithmetic expression
set bintflux=`calc -f f8.2 "$bflux*($delv)"`
echo "Flux in the cleaned CARMA cube is:  $bintflux Jy km/s" 
set kflux=`getflux in=$sdmap.gain region=$emreg`
set kintflux=`calc -f f8.2 "$kflux*($delv)"`
echo "Flux in the calibrated SD cube is: $kintflux Jy km/s" 

minmax in=$sdmap.gain
minmax in=$intcm.$sdbmsz 
set crms=`getrms region="$sdrmsreg" in=$sdmap.gain`
echo "RMS noise in calibrated SD cube is $crms"

#goto Combine

Display:
echo "Contours are singledish, greyscale is CARMA"
cgdisp in=$sdmap.gain,$intcm.$sdbmsz type=c,g options=full,mirr,3val,abut,noepoch \
        labtyp=arcsec nxy=$nxy csize=1,1 slev=a,$crms \
	    levs1=4,8,12,16,20,24,28,32,36,40 \
        device=$device range=-5,35,lin,1 cols1=6 3format='f5.2'

# --------------------------------------------------------------------------
# 2. Perform the combination.
# --------------------------------------------------------------------------

Combine:
# Now do the linear combination 
set carmamin=`gethd in=$intcm/bmin`
set carmamaj=`gethd in=$intcm/bmaj`
set carmaba=`calc -f f9.5 "(206265**2)*$carmamin*$carmamaj"`
set alpha_rec=`calc -f f7.4 "$carmaba/($sdbmsz*$sdbmsz)"`
echo "Using ratio of beam areas $alpha_rec for relative weighting factor"
set alpha=$alpha_rec
\rm -rf $of.map $of.beam $of.map.masked $of.map.masked1
maths exp="(<$intmap>+$alpha*<$sdmap.gain>)/(1.+$alpha)" out=$of.map
maths exp="(<$intbeam>+$alpha*<$sdbeam.trunc>)/(1.+$alpha)" out=$of.beam
# mask the combined map to remove low sensitivity edges
maths exp="<$of.map>" mask="<$sen>.lt.$mask" out="$of.map.masked1"


#goto Deconvolve

echo "Adding $sdmap.gain mask to $of.map.masked1"
immask in=$of.map.masked1  logic=OR region=mask"($sdmap.gain)"
imsub in=$of.map out=$of.map.masked region=mask"($of.map.masked1)"

minmax in=$of.map.masked
minmax in=$of.beam
echo "Linearly combined, masked, dirty+SD map"
cgdisp in=$of.map.masked device=$device nxy=$nxy labtyp=arcsec \
	options=3val,abut,noepoch 3format='f5.2'

# --------------------------------------------------------------------------
# 3. Deconvolve the combined map.
# --------------------------------------------------------------------------

Deconvolve:

# Deconvolve with mossdi
\rm -rf $of.sdi
echo "MOSSDI began on `date`" 
echo "Will clean to cutoff=$cutoff or niters=$niters"
mossdi map=$of.map.masked beam=$of.beam out=$of.sdi \
        cutoff=$cutoff niters=$niters gain=$cleangain
echo "MOSSDI completed on `date`"
minmax in=$of.sdi

Restore:
\rm -rf $psf
mospsf beam=$of.beam out=$psf
minmax in=$psf
\rm -rf $of.r
restor map=$of.map.masked beam=$psf model=$of.sdi out=$of.r
\rm -rf $of.residual
restor map=$of.map.masked beam=$psf model=$of.sdi out=$of.residual mode=residual
copyhd in=$of.r out=$of.residual items=bmaj,bmin,bpa
minmax in=$of.r
minmax in=$of.r

set delv=`gethd in=$of.r/cdelt3`
set cflux=`getflux in=$of.r region=$emreg`
set cintflux=`calc -f f8.2 "$cflux*$delv"`
echo "Flux in the combined cube is: $cintflux Jy km/s" 

# Plot sensitivity-normalized residuals, and cleaned maps:
set minval=`getmin in=$sen region='image(1)'`
\rm -rf $of.residual.sen
maths exp="(<$of.residual>*$minval)/<$sen>" out=$of.residual.sen
minmax in=$of.residual.sen

echo " Displaying sensitivity-normalized residuals"
cgdisp in=$of.residual.sen device=$device csize=1,1 nxy=$nxy \
   options=full,3val,abut,noepoch labtyp=arcsec 3format='f5.2'
echo " Displaying restored map"
cgdisp in=$of.r device=$device type=p csize=1,1 nxy=$nxy \
   options=full,3val,abut,noepoch labtyp=arcsec 3format='f5.2'

# --------------------------------------------------------------------------
# 4. Evaluate the results.
# --------------------------------------------------------------------------

Evaluate:

# Look at integrated spectra:
echo "Comparing spectra for combined data, CARMA, and SD..."
imstat in=$of.r region=$emreg plot=mean device=combspec.ps/ps options=tb
ghostview combspec.ps&
##lpr combspec.ps 
imstat in=$intcm region=$emreg plot=mean device=carmaspec.ps/ps options=tb
##lpr carmaspec.ps 
ghostview carmaspec.ps&
imstat in=$sdmap.gain region=$emreg plot=mean device=sdspec.ps/ps options=tb
##lpr sdspec.ps 
ghostview sdspec.ps&

#goto TheEnd

set device=/null
Plot:
# Plot difference between combined and CARMA-only maps:
echo "Calculating difference between combined and CARMA-only maps..."
\rm -rf $diff.r
maths exp="<$of.r>-<$intcm>" out=$diff.r
set crms=`getrms in=$intcm region="$intrmsreg"`
echo "RMS noise in $intcm is $crms"
cgdisp in=$diff.r,$intcm type=p,c options=full,3value,mirr,abut,noepoch labtyp=arcsec \
    	slev=a,$crms levs1=2,3,4,5,6,7,8,9,10,11,12,13,14,15,16 \
	csize=1,1 device=$device nxy=$nxy cols1=0 3format='f5.2'

# Convolve the combined map to $sdbmsz" and compare to SD map:
echo "Convolving combined map to SD resolution..."
\rm -rf $of.$sdbmsz
convol map=$of.r fwhm=$sdbmsz pa=0 options=final out=$of.$sdbmsz
set crms=`getrms in=$sdmap.gain region=$sdrmsreg`
echo "RMS noise in $sdmap.gain is $crms"
echo "SD data is contour, combined map is color"
cgdisp in=$sdmap.gain,$of.$sdbmsz type=c,p nxy=$nxy options=full,mirr,3val,abut,noepoch \
	labtyp=arcsec csize=1,1 slev=a,$crms levs1=2,4,6,8,10,12,14,16,18,20,22,24 cols1=6 \
	device=$device 3format='f5.2'

# Compare combined and CARMA map at $sdbmsz" resolution:
set crms=`getrms in=$intcm.$sdbmsz region="$intrmsreg"`
echo "RMS noise in $intcm.$sdbmsz is $crms"
echo "CARMA data is contour, combined map is color"
cgdisp in=$intcm.$sdbmsz,$of.$sdbmsz type=c,p nxy=$nxy options=full,mirr,3val,abut,noepoch \
	labtyp=arcsec csize=1,1 slev=a,$crms levs1=2,4,6,8,10,12,14,16,18,20,22,24 cols1=6 \
	device=$device 3format='f5.2'

goto TheEnd

TheEnd:
pipepar -f lincomb.def intmap=$origintmap intbeam=$intbeam intcm=$origintcm sdmap=$sdmap sdbmsz=$sdbmsz jyperk=$jyperk device=$device chan=$chan niters=$niters mask=$mask nxy=$nxy
exit 0

Intrupt:
echo '####### Interrupted -- OK: Cleaning up ######'
#\rm -rf $sdbeam $sdbeam.trunc
#\rm -rf $regriddedmap $sdmap.jy
#\rm -rf $intcm.$sdbmsz
#\rm -rf $gain $sen $psf

exit 1

Help:
    echo "${B}$myname${N} performs joint deconvolution of interferometric and "
    echo "singledish maps using the method outlined in "
    echo "Stanimirovic et al. 1999, MNRAS, 302, 417"
    echo "Briefly, the dirty map and singledish map are added together,"
    echo "weighted by the ratio of their beam areas.  The beams are similarly"
    echo "added and the combined map is deconvolved using the combined "
    echo "beam with MOSSDI.  The deconvolution will clean down to twice"
    echo "the RMS noise in the interferometric dirty map or until the"
    echo "maximum number of iterations is performed. "
    echo " "
    echo "${B}$myname${N} takes command line parameters in key=value format."
    echo "Some parameters are required, others are optional."
    echo "If mis.def is in the current working directory, parameters are read"
    echo "from there before parsing the command line."
    echo "Parameters:"
    echo "${B}-h${N}      : List this help"
    echo "${B}intmap ${N} : The interferometric dirty map. No default"
    echo "${B}intbeam${N} : The interferometric dirty beam. No default"
    echo "${B}intcm${N}   : The cleaned and restored interferometric map. No default"
    echo "${B}sdmap${N}   : The singledish map, in units of brightness temperature. "
    echo "          This map will be multiplied by by jyperk and regridded "
    echo "          to match the interferometric map. No default."
    echo "${B}jyperk${N}  : The Jy/K by which to scale the singledish map. No default. This is equivalent to changing 'f' in the combination equation"
    echo "${B}sdbmsz${N}  : The singledish beam size in arcseconds.  If not given, "
    echo "          it will be computed from the 'telescop' and 'restfreq' "
    echo "          parameters in the sdmap image file."
    echo "${B}chan${N}    : start and end channels to use, e.g. chan=5,20. "
    echo "          Default is all channels"
    echo "${B}niters${N}  : Maximum  number of MOSSDI iterations to perform per plane."
    echo "          Default: 100 (usually a much larger number is needed)"
    echo "${B}cutoff${N}  : Mossdi cutoff.  Default is twice the 1 sigma RMS "
    echo "                : in the restored interferometric map (intcm)"
    echo "                : Default: twice the rms in the dirty int. map"
    echo "${B}mask${N}    : Sensitivity in Jy/beam above which to mask the dirty"
    echo "          combined image BEFORE deconvolution. This prevents MOSSDI"
    echo "          from finding components near the edges of the map. "
    echo "          Default: 0.5 Jy/beam" 
    echo "${B}device${N}  : PGPLOT device. Default: /null"
    echo "${B}nxy${N}     : Number of sub-plots in the x and y directions on the page,"
    echo "          for CGDISP.  Default: 4,4"
    echo " "

Usage:
    echo "${B}Usage:${N} "
    echo "${B}$myname -h${N}  for help"
    echo "${B}$myname${N} intmap=[dirtymap] intbeam=[dirtybeam] intcm=[restored map] sdmap=[singledish map] jyperk=[Jy/K] [ sdbmsz=[SD beamsize in arcsec] ] [device= ] [chan=startchan,endchan] [niters=number of interations] mask=[sensitivity level above which to mask the dirty combined image, Jy/beam"
    exit 1

