#!/bin/tcsh -f
#
# Script to combine Singledish and CARMA data using mosmem
# $Id: memcomb.csh,v 1.8 2012/03/26 20:20:06 mpound Exp $

set path = ( $MIS/bin $MIRBIN $path )

set B="\033[1m"   # bold
set N="\033[0m"  # normal
set myname=`basename $0` 

onintr Intrupt

# dirty map, dirty beam, single dish map 
unset intmap  # dirty map
unset intbeam # dirty beam
unset intcm
# Input singledish map, typically in temperature units.
# This script will regrid and convert to Janskys using jyperk keyword
unset sdmap
unset sdbmsz
unset jyperk
unset chan

# default display parameters
set device=/xs
set nxy=4,4

# default MOSMEM iterations and rmsfac
set niters=30
set rmsfac=1.1,1.1

# Region over which to measure flux and rms
# NOTE: These are only good for HCN test map!!!
set emreg="arcsec,box(-250,-400,200,400)"
# interferometric
set intrmsreg="image(1,5)"

# pipeline can override them, and grab old variable
pipepar -f memcomb.def -c -s csh > tmp$$.par;  source tmp$$.par;  rm tmp$$.par

# command line overrides (just a cute CSH trick)
foreach _arg ($*)
  if ( $_arg == "-h" ) goto Help
  set $_arg
end

if ($?intmap == 0 ) then
  echo intmap must be given 
  goto Usage
endif
if ($?intbeam == 0 ) then
  echo intbeam must be given 
  goto Usage
endif
if ($?sdmap == 0 ) then
  echo sdmap must be given 
  goto Usage
endif
if ($?jyperk == 0 ) then
  echo jyperk must be given 
  goto Usage
endif

if ($?sdbmsz == 0 ) then
  echo Singledish beamsize will be computed from the data
endif

echo "Starting $0 $* at `date`"

#goto Restore
# --------------------------------------------------------------------------
# 1. Prepare files for combination.
# --------------------------------------------------------------------------

# Compute the sdbmsz from the telescope and 
# frequency if it was not given on the command line.
if ( $?sdbmsz == 0 ) then 
  set telescop=`itemize in=$intmap/telescop | tail -1 |awk '{print $3}'`
  set restfreq=`itemize in=$intmap/restfreq | tail -1 |awk '{print $3}'`
  set sdbmsz=`pbplot telescop=$telescop freq=$restfreq | grep FWHM| awk '{print $3*60.0}'`
  echo "Computed beamsize is $sdbmsz arcsec"
else 
  echo "Using beamsize $sdbmsz arcsec"
endif

if ( $?sdrms == 0 ) then
   set sdrms=`itemize in=$sdmap/rms|tail -1 |awk '{print $3}'`
   if ( $sdrms == "data,") then
      echo "No RMS found in singledish map. Please provide it on command line"
      echo "or use puthd to insert in the the map header."
      goto Usage
   endif
else 
   puthd in=$sdmap/rms value=$sdrms type=d
endif
   
set origintmap=$intmap
if ( $?intcm != 0 ) then 
    set origintcm=$intcm
endif


# Use MIRIAD regrid and maths tasks to regrid input singledish map
# Add some header variables to the singledish map
# !!!!MUST REGRID BEFORE IMSUB!!!!  because SD map might have
# different number of channels than intmap.
set regriddedmap=$sdmap.regrid
/bin/rm -rf $regriddedmap $sdmap.jy
set naxis=`itemize in=$intmap/naxis|tail -1 |awk '{print $3}'`
# make fake 4th axis if necessary
if ( $naxis == 3 ) then
   bigger $sdmap
endif
regrid tin=$intmap in=$sdmap axes=1,2,3 out=$regriddedmap
maths exp='<'$regriddedmap'>*'$jyperk out=$sdmap.jy
puthd in=$sdmap.jy/bmaj value=$sdbmsz,arcsec type=d
puthd in=$sdmap.jy/bmin value=$sdbmsz,arcsec type=d
puthd in=$sdmap.jy/bpa  value=0 type=d
puthd in=$sdmap.jy/rms value=$sdrms type=d
minmax in=$sdmap.jy


set sdmapfile=$sdmap.jy
set sdbeamfile=sdbeam
if ( $?chan != 0 ) then
  set chext = `echo $chan |tr ',' '_'`
  echo "Grabbing channels $chan from maps"
  rm -rf $intmap.$chext  
  imsub in=$intmap out=$intmap.$chext region=image'('$chan')'
  set intmap=$intmap.$chext
  minmax in=$intmap
  /bin/rm -rf $sdmap.$chext.jy
  imsub in=$sdmap.jy out=$sdmap.$chext.jy region=image'('$chan')'
  minmax in=$sdmap.$chext.jy
  set sdmapfile=$sdmap.$chext.jy
  set sdbeamfile=sdbeam.$chext
  if ( $?intcm != 0 ) then
      rm -rf $intcm.$chext  
      imsub in=$intcm out=$intcm.$chext region=image'('$chan')'
      set intcm=$intcm.$chext
      minmax in=$intcm
  endif
endif

#--------------------------------------------
# Set output names after final intmap set.
#
# Output of mospsf -- used in restore step
set psf=$intmap.psf
# output of mossen
set gain=$intmap.gain
set sen=$intmap.sen

# linear combination output
set of=$intmap.memcomb
# difference map
set diff=$of.diff
#--------------------------------------------

#
# Generate singledish beam and truncate it at 5% level.
#
echo "Creating SD beam and truncating at 5% level"
\rm -rf $sdbeamfile
\rm -rf beam1
imsub in=$intbeam region='image(1)' out=beam1
imgen in=beam1 factor=0 out=$sdbeamfile spar=1,0,0,$sdbmsz,$sdbmsz,0
\rm -rf $sdbeamfile.trunc
maths exp='<'$sdbeamfile'>' mask='<'$sdbeamfile'>'.ge.0.05 out=$sdbeamfile.trunc
minmax in=$sdbeamfile.trunc
# singledish beam file
set tmp=$sdbeamfile.trunc
set sdbeamfile=$tmp

#echo "Applying CARMA gain to Singledish map..."
#

ShowSize:
echo "$sdmapfile size:"
itemize in=$sdmapfile/naxis1
itemize in=$sdmapfile/naxis2
itemize in=$sdmapfile/naxis3
#echo 'gain size:'
#
#itemize in=$gain/naxis2
#itemize in=$gain/naxis3


#exit
# --------------------------------------------------------------------------
# 3. Deconvolve the combined map.
# --------------------------------------------------------------------------

Deconvolve:

# Deconvolve with mossdi
\rm -rf $of.sdi $of.mem
mosmem map=$intmap,$sdmapfile beam=$intbeam,$sdbeamfile out=$of.mem \
        niters=$niters rmsfac=$rmsfac
minmax in=$of.mem

Restore:
\rm -rf $psf
mospsf beam=$intbeam out=$psf
minmax in=$psf
\rm -rf $of.r
restor map=$intmap beam=$psf model=$of.mem out=$of.r
\rm -rf $of.residual
restor map=$intmap beam=$psf model=$of.mem out=$of.residual mode=residual
copyhd in=$of.r out=$of.residual items=bmaj,bmin,bpa
minmax in=$of.r
minmax in=$of.residual

set delv=`gethd in=$of.r/cdelt3`
set cflux=`getflux in=$of.r region=$emreg`
set cintflux=`calc -f f8.2 "$cflux*$delv"`
echo "Flux in the combined cube is: $cintflux Jy km/s" 

#
# Weight Single dish map by CARMA gain
#
/bin/rm -rf $gain $sen
mossen in=$intmap gain=$gain sen=$sen

# Plot sensitivity-normalized residuals, and cleaned maps:
set minval=`getmin in=$sen region='image(1)'`
\rm -rf $of.residual.sen
maths exp="(<$of.residual>*$minval)/<$sen>" out=$of.residual.sen
minmax in=$of.residual.sen

echo " Displaying sensitivity-normalized residuals"
cgdisp in=$of.residual.sen device=$device csize=1,1 nxy=$nxy \
   options=full,3val,abut,noepoch labtyp=arcsec 3format='f5.2'
echo " Displaying restored map"
cgdisp in=$of.r device=$device type=p csize=1,1 nxy=$nxy \
   options=full,3val,abut,noepoch labtyp=arcsec 3format='f5.2'

# --------------------------------------------------------------------------
# 4. Evaluate the results.
# --------------------------------------------------------------------------

if ( $?intcm == 0 ) goto TheEnd

Evaluate:

# Look at integrated spectra:
echo "Comparing spectra for combined data, CARMA, and SD..."
imstat in=$of.r region=$emreg plot=mean device=combspec.ps/ps options=tb
ghostview combspec.ps&
imstat in=$intcm region=$emreg plot=mean device=carmaspec.ps/ps options=tb
ghostview carmaspec.ps&

#goto TheEnd

Plot:
# Plot difference between combined and CARMA-only maps:
echo "Calculating difference between combined and CARMA-only maps..."
\rm -rf $diff.r
maths exp="<$of.r>-<$intcm>" out=$diff.r
set crms=`getrms in=$intcm region="$intrmsreg"`
echo "RMS noise in $intcm is $crms"
cgdisp in=$diff.r,$intcm type=p,c options=full,3value,mirr,abut,noepoch labtyp=arcsec \
    	slev=a,$crms levs1=2,3,4,5,6,7,8,9,10,11,12,13,14,15,16 \
	csize=1,1 device=$device nxy=$nxy cols1=0 3format='f5.2'

# Convolve the combined map to $sdbmsz" and compare to SD map:
echo "Convolving combined map to SD resolution..."
\rm -rf $of.$sdbmsz
convol map=$of.r fwhm=$sdbmsz pa=0 options=final out=$of.$sdbmsz
echo "SD data is contour, combined map is color"
cgdisp in=$sdmapfile,$of.$sdbmsz type=c,p nxy=$nxy options=full,mirr,3val,abut,noepoch \
	labtyp=arcsec csize=1,1 slev=a,$sdrms levs1=2,4,6,8,10,12,14,16,18,20,22,24 cols1=6 \
	device=$device 3format='f5.2'

# Compare combined and CARMA map at $sdbmsz" resolution:
set crms=`getrms in=$intcm.$sdbmsz region="$intrmsreg"`
echo "RMS noise in $intcm.$sdbmsz is $crms"
echo "CARMA data is contour, combined map is color"
cgdisp in=$intcm.$sdbmsz,$of.$sdbmsz type=c,p nxy=$nxy options=full,mirr,3val,abut,noepoch \
	labtyp=arcsec csize=1,1 slev=a,$crms levs1=2,4,6,8,10,12,14,16,18,20,22,24 cols1=6 \
	device=$device 3format='f5.2'

goto TheEnd

TheEnd:
if ( $?origintcm == 0 ) then 
    pipepar -f memcomb.def intmap=$origintmap intbeam=$intbeam sdmap=$sdmap sdbmsz=$sdbmsz jyperk=$jyperk device=$device chan=$chan niters=$niters nxy=$nxy rmsfac=$rmsfac sdrms=$sdrms
else 
    pipepar -f memcomb.def intmap=$origintmap intbeam=$intbeam intcm=$origintcm sdmap=$sdmap sdbmsz=$sdbmsz jyperk=$jyperk device=$device chan=$chan niters=$niters nxy=$nxy rmsfac=$rmsfac sdrms=$sdrms
endif
exit 0

Intrupt:
echo "####### Interrupted -- OK: Cleaning up ######"
#\rm -rf $sdbeam $sdbeam.trunc
#\rm -rf $regriddedmap $sdmap.jy
#\rm -rf $intcm.$sdbmsz
#\rm -rf $gain $sen $psf

exit 1

Help:
    echo "${B}$myname${N} performs joint deconvolution of interferometric and "
    echo "singledish maps using MOSMEM"
    echo " "
    echo "${B}$myname${N} takes command line parameters in key=value format."
    echo "Some parameters are required, others are optional."
    echo "If mis.def is in the current working directory, parameters are read"
    echo "from there before parsing the command line."
    echo "Parameters:"
    echo "${B}-h${N}      : List this help"
    echo "${B}intmap ${N} : The interferometric dirty map. No default"
    echo "${B}intbeam${N} : The interferometric dirty beam. No default"
    echo "${B}intcm${N}   : The cleaned and restored interferometric map. Optional, no default. If given, a comparison between the cleaned maps will be made, otherwise it will be skipped."
    echo "${B}sdmap${N}   : The singledish map, in units of brightness temperature. "
    echo "          This map will be multiplied by by jyperk and regridded "
    echo "          to match the interferometric map. No default."
    echo "${B}sdrms${N}   : The singledish rms, in units of Jy/beam, if not "
    echo "                : already present in the sdmap image header. No default."
    echo "${B}sdbmsz${N}  : The singledish beam size in arcseconds.  If not given, "
    echo "          it will be computed from the 'telescop' and 'restfreq' "
    echo "          parameters in the sdmap image file."
    echo "${B}jyperk${N}  : The Jy/K by which to scale the singledish map. No default."
    echo "${B}chan${N}    : start and end channels to use, e.g. chan=5,20. "
    echo "          Default is all channels"
    echo "${B}niters${N}  : Maximum  number of MOSMEM iterations to perform per plane."
    echo "          Default: 30"
    echo "${B}rmsfac${N}  : MOSMEM rmsfac parameter. One or two values for interferometric and singledish respectively. Default: 1.1,1.1"
    echo "${B}device${N}  : PGPLOT device. Default: /null"
    echo "${B}nxy${N}     : Number of sub-plots in the x and y directions on the page,"
    echo "          for CGDISP.  Default: 4,4"
    echo " "

Usage:
    echo "${B}Usage:${N} "
    echo "${B}$myname -h${N}  for help"
    echo "${B}$myname${N} intmap=[dirtymap] intbeam=[dirtybeam] intcm=[restored map] sdmap=[singledish map] jyperk=[Jy/K] [sdrms=[SD noise in Jy/beam]] [ sdbmsz=[SD beamsize in arcsec] ] [device= ] [chan=startchan,endchan] [niters=number of interations] [rmsfac=]"
    exit 1

