#!/bin/csh -f
echo "   ---  ALMA Mosaicing  ---   "
echo "  $0   mchw. 22jul2011 version.  `date`"

# History:
#  june 02 mchw. ALMA script.
#  15aug02 mchw. CARMA version edited from ALMA script.
#  23aug02 mchw. calculate region from source size.
#  20sep02 mchw. Re-import CARMA improvements for ALMA.
#  25sep02 mchw. Re-import improvements from hex7.csh to hex19.csh
#  26sep02 mchw. Increase imsize from 129 to 257.
#  24jun10 mchw. Added uv and image plots.
#  13jul10 mchw. Added default and joint deconvolutions.
#  16jul10 mchw. Added gif plots. Changed rmsfac=200 to 1, same as carma version
#  07jul11 mchw. hex1.csh edit from hex19.csh.
#  22jul11 mchw. uvmodel.csh

goto start
start:

# check inputs
  if($#argv<4) then
    echo " Usage:  $0 config freq cell "
    echo "   e.g.  $0 config1 345 0.175"
    echo " Inputs :"
    echo "   array"
    echo "          Antenna array used by uvgen task."
    echo "            e.g. config1. Omit .ant.  No default."
    echo "   freq"
    echo "          frequency. GHz. No default."
    echo "   cell"
    echo "          image pixel in arcsec, for imaging and scaling model size. No default."
    echo "   method"
    echo "          Deconvolution method: clean, mosmem, default,joint or plot. Default is clean."
    echo "          default and joint methods use the uvdata and images generated by mosmem."
    echo "          plot -- re-plots the last result"

    echo " "
    exit 1
  endif

# Cas A model, cas.vla, pixel=0.4", Cas A is about 320" diameter; imsize=1024 == 409.6"
# scale model size. eg. cell=0.1 arcsec -> 80" diameter

# Saturn model, sat1mm.modj2, pixel=0.1", Saturn's rings are ~ 50" diameter; imsize=603 == 60"

set model   = sat1mm.modj2
set model   = Halo3.mp
set model   = casc.vla
set model   = sgra3.5cm.icln
set model   = sgra1.3cm.icln
set config  = $1
set freq    = $2
set cell    = $3
set method  = $4
# Nyquist sample rate for each pointing.
calc '6/(pi*250)*12'
set harange = -4,4,.1
set ra      = '17:45:40.051'
set dec     = '-29:00:27.978'  
set select  = '-shadow(12)'
set nchan   = 1
set imsize  = 257
set imsize  = 1025
set imsize  = 2049
set region  = quart

echo "Mosaicing with ALMA  $0 $1 $2 $3 $4 $model,  `date`   " >> $0.$model.results
echo "Mosaicing with ALMA  $0 $1 $2 $3 $4 $model,  `date`   " > timing
echo " model   = $model"             >> timing
echo " config  = $config"            >> timing
echo " freq    = $freq"              >> timing
echo " ra      = $ra"                >> timing
echo " dec     = $dec"               >> timing
echo " harange = $harange  hours"    >> timing
echo " cell    = $cell"              >> timing
echo " select  = $select"            >> timing
echo " nchan   = $nchan"             >> timing
echo " imsize  = $imsize"            >> timing
echo " region  = $region"            >> timing
echo " method  = $method"            >> timing
echo " "                             >> timing
echo "   ---  TIMING   ---   "       >> timing
echo START: `date` >> timing

if($method == plot) then
  goto plot
endif

if($method == default) then
  goto default
endif

if($method == joint) then
  goto joint
endif

echo "Generate mosaic grid"
#  lambda/2*antdiam (arcsec)
calc "300/$freq/2/12e3*2e5"

echo "Generate uv-data. Tsys=40K, bandwidth=8 GHz " >> timing
rm -r $config.uv
uvgen ant=$config.ant baseunit=-3.33564 radec=$ra,$dec lat=-23.02 harange=$harange source=$MIRCAT/no.source systemp=40 jyperk=40 freq=$freq corr=$nchan,1,0,8000 out=$config.uv telescop=alma 
echo UVGEN: `date` >> timing
uvindex vis=$config.uv

# scale model size. eg. for CasA, cell=0.1" gives 80" diam, cell=.01" gives-> 8" diam
# with 0.4 arcsec pixel size Cas A ~ 320 arcsec diameter; image size 1024 == 409.6"
# make RA, DEC, and FREQ in model agree with the uvdata generated by uvgen.

echo "Scale model size to $cell arcsec pixel" >> timing
rm -r single.$freq.$model.$cell
cp -r $model single.$freq.$model.$cell
puthd in=single.$freq.$model.$cell/crval1 value=$ra,hms
puthd in=single.$freq.$model.$cell/crval2 value=$dec,dms
puthd in=single.$freq.$model.$cell/crval3 value=$freq
puthd in=single.$freq.$model.$cell/cdelt1 value=-$cell,arcsec
puthd in=single.$freq.$model.$cell/cdelt2 value=$cell,arcsec

echo "Make model images for each pointing center" >> timing
rm -rf $config.$freq.$model.$cell.demos*
demos map=single.$freq.$model.$cell vis=$config.uv out=$config.$freq.$model.$cell.demos

echo "Make uv-data using model image (the model may have the primary beam)" >> timing
  rm -r $config.$freq.$model.$cell.uv*
 foreach i (1)
    cgdisp device=/xs labtyp=arcsec range=0,0,lin,8 in=$config.$freq.$model.$cell.demos$i
    uvmodel vis=$config.uv model=$config.$freq.$model.$cell.demos$i options=add,selradec out=$config.$freq.$model.$cell.uv$i line=chan,1,1,1,1
    uvplt vis=$config.$freq.$model.$cell.uv$i axis=uvd,amp options=nobase device=/xs
  end
echo UVMODEL: `date` add the model to the noisy sampled uv-data >> timing

uvplt vis=$config.$freq.$model.$cell.uv1 axis=uc,vc options=equal,nobase device=/xs
uvplt vis=$config.$freq.$model.$cell.uv1 axis=uc,vc options=equal,nobase device=uvplt.$config.$freq.$model.$cell.gif/gif

echo "Make model images for each pointing center" >> timing
# INVERT uses the same cell size as the model image.
rm -r $config.$freq.$model.$cell.mp $config.$freq.$model.$cell.bm
invert "vis=$config.$freq.$model.$cell.uv*" map=$config.$freq.$model.$cell.mp beam=$config.$freq.$model.$cell.bm imsize=$imsize cell=$cell sup=0 options=mosaic,double select=$select
echo INVERT: `date` >> timing
implot in=$config.$freq.$model.$cell.mp device=/xs units=s region=$region
imlist in=$config.$freq.$model.$cell.mp options=mosaic

goto clean

joint:
echo "Joint deconvolution of interferometer and single dish data" >> timing
echo "Joint deconvolution of interferometer and single dish data ; niters=200 rmsfac=1,1" >> $0.$model.results
rm -r $config.$freq.$model.$cell.mem $config.$freq.$model.$cell.cm
 mosmem  map=$config.$freq.$model.$cell.mp,single.$freq.$model.$cell.map beam=$config.$freq.$model.$cell.bm,single.$freq.$model.$cell.beam out=$config.$freq.$model.$cell.mem niters=200 region=$region rmsfac=1,1
goto restor
 
mosmem:
echo " MOSMEM Interferometer only" >> timing
echo " MOSMEM Interferometer only with niters=200 flux=732.063 rmsfac=1." >> $0.$model.results
rm -r $config.$freq.$model.$cell.mem $config.$freq.$model.$cell.cm
mosmem  map=$config.$freq.$model.$cell.mp beam=$config.$freq.$model.$cell.bm out=$config.$freq.$model.$cell.mem region=$region niters=200 flux=732.063 rmsfac=1
goto restor

default:
echo "MOSMEM with default single dish image"  >> timing
echo "MOSMEM with default single dish image; niters=200 rmsfac=1"  >> $0.$model.results
rm -r $config.$freq.$model.$cell.mem $config.$freq.$model.$cell.cm
mosmem map=$config.$freq.$model.$cell.mp default=single.$freq.$model.$cell.map beam=$config.$freq.$model.$cell.bm out=$config.$freq.$model.$cell.mem region=$region niters=200 rmsfac=1
goto restor

clean:
echo CLEAN: `date` >> timing
rm -r $config.$freq.$model.$cell.mem $config.$freq.$model.$cell.cm
clean map=$config.$freq.$model.$cell.mp beam=$config.$freq.$model.$cell.bm out=$config.$freq.$model.$cell.mem  niters=1000

restor:
restor map=$config.$freq.$model.$cell.mp beam=$config.$freq.$model.$cell.bm out=$config.$freq.$model.$cell.cm model=$config.$freq.$model.$cell.mem
implot device=/xs units=s in=$config.$freq.$model.$cell.cm
goto convolve

rm -r $config.$freq.$model.$cell.memrem
restor map=$config.$freq.$model.$cell.mp beam=$config.$freq.$model.$cell.bm out=$config.$freq.$model.$cell.memrem model=$config.$freq.$model.$cell.mem mode=residual
implot device=/xs units=s in=$config.$freq.$model.$cell.mem region=$region
mem_residual:
histo in=$config.$freq.$model.$cell.memrem region=$region | grep Rms
goto end

convolve:
echo "convolve the model by the beam and subtract from the deconvolved image" >> timing
  set b1=`prthd in=$config.$freq.$model.$cell.cm | egrep Beam     | awk '{print $3}'`
  set b2=`prthd in=$config.$freq.$model.$cell.cm | egrep Beam     | awk '{print $5}'`
  set b3=`prthd in=$config.$freq.$model.$cell.cm | egrep Position | awk '{print $3}'`
  rm -r $config.$freq.$model.$cell.conv
  convol map=single.$freq.$model.$cell fwhm=$b1,$b2 pa=$b3 out=$config.$freq.$model.$cell.conv
  implot in=$config.$freq.$model.$cell.conv device=/xs units=s region=$region
echo "regrid the convolved model to the deconvolved image template" >> timing
  rm -r $config.$freq.$model.$cell.regrid
  regrid in=$config.$freq.$model.$cell.conv out=$config.$freq.$model.$cell.regrid tin=$config.$freq.$model.$cell.cm axes=1,2
  implot in=$config.$freq.$model.$cell.regrid device=/xs units=s region=$region

  cgdisp range=0,0,lin,8 in=$config.$freq.$model.$cell.cm device=$config.$freq.$model.$cell.cm.gif/gif labtyp=arcsec,arcsec options=beambl,wedge  region=$region device=/xs
  cgdisp range=0,0,lin,8 in=$config.$freq.$model.$cell.cm device=$config.$freq.$model.$cell.cm.gif/gif labtyp=arcsec,arcsec options=beambl,wedge  region=$region
  cgdisp range=0,0,lin,8 in=$config.$freq.$model.$cell.regrid device=$config.$freq.$model.$cell.regrid.gif/gif labtyp=arcsec,arcsec options=beambl,wedge  region=$region device=/xs
  cgdisp range=0,0,lin,8 in=$config.$freq.$model.$cell.regrid device=$config.$freq.$model.$cell.regrid.gif/gif labtyp=arcsec,arcsec options=beambl,wedge  region=$region
rm -r $config.$freq.$model.$cell.imcat
imcat in=$config.$freq.$model.$cell.cm,$config.$freq.$model.$cell.regrid out=$config.$freq.$model.$cell.imcat options=relax
 cgdisp range=0,0,lin,8 in=$config.$freq.$model.$cell.imcat device=$config.$freq.$model.$cell.imcat.gif/gif labtyp=arcsec,arcsec options=beambl,wedge  region=$region device=/xs
 cgdisp range=0,0,lin,8 in=$config.$freq.$model.$cell.imcat device=$config.$freq.$model.$cell.imcat.gif/gif labtyp=arcsec,arcsec options=beambl,wedge region=$region

echo "compare the model with the deconvolved image " >> timing
  rm -r $config.$freq.$model.$cell.resid
  imdiff in1=$config.$freq.$model.$cell.cm in2=$config.$freq.$model.$cell.regrid resid=$config.$freq.$model.$cell.resid options=nox,noy,noex
  implot device=/xs units=s in=$config.$freq.$model.$cell.resid region=$region
  histo in=$config.$freq.$model.$cell.resid region=$region

plot:
echo "plot the model and deconvolved image " >> timing
rm -r $config.$freq.$model.$cell.imcat
imcat in=$config.$freq.$model.$cell.cm,$config.$freq.$model.$cell.regrid options=relax out=$config.$freq.$model.$cell.imcat
cgdisp range=0,0,lin,8 in=$config.$freq.$model.$cell.imcat labtyp=arcsec,arcsec options=beambl,wedge region=$region device=/xs
cgdisp range=0,0,lin,8 in=$config.$freq.$model.$cell.imcat labtyp=arcsec,arcsec options=beambl,wedge region=$region device=$config.$freq.$model.$cell.imcat.gif/gif

if($method == plot) then
  goto end
endif

echo FINISH: `date` >> timing
echo " " >> timing

echo print out results - summarize rms and beam sidelobe levels
echo "   ---  RESULTS   ---   " >> timing
set RMS = `itemize in=$config.$freq.$model.$cell.mp   | grep rms       | awk '{printf("%.3f   ", 1000*$3)}'`
set SRMS = `histo in=$config.$freq.$model.$cell.resid region=$region | grep Rms | awk '{printf("%.3f   ", $4)}'`
set SMAX = `histo in=$config.$freq.$model.$cell.resid region=$region | grep Maximum | awk '{printf("%.3f   ", $3)}'`
set SMIN = `histo in=$config.$freq.$model.$cell.resid region=$region | grep Minimum | awk '{printf("%.3f   ", $3)}'`
set Model_Flux = `histo in=$config.$freq.$model.$cell.conv region=$region | grep Flux | awk '{printf("%.3f   ", $6)}'`
set Model_Peak = `histo in=$config.$freq.$model.$cell.conv region=$region | grep Maximum | awk '{printf("%.3f   ", $3)}'`
set Flux = `histo in=$config.$freq.$model.$cell.cm region=$region | grep Flux | awk '{printf("%.3f   ", $6)}'`
set Peak = `histo in=$config.$freq.$model.$cell.cm region=$region | grep Maximum | awk '{printf("%.3f   ", $3)}'`
set Fidelity = `calc $Peak/$SRMS | awk '{printf("%.0f", $1)}'`

echo "Config  freq[GHz]  HA[hrs]  RMS[mJy] Beam[arcsec] cell Model_Flux,Peak Image_Flux,Peak Residual:Rms,Max,Min[Jy] Fidelity" >> timing
echo  "$config  $freq  $harange  $RMS   $b1 $b2    $cell  $Model_Flux $Model_Peak  $Flux $Peak   $SRMS  $SMAX  $SMIN  $Fidelity" >> timing
echo  " "
echo  "$config  $freq  $harange  $RMS   $b1 $b2    $cell  $Model_Flux $Model_Peak  $Flux $Peak   $SRMS  $SMAX  $SMIN  $Fidelity" >> $0.$model.results
mv timing $0.$config.$freq.$harange.$nchan.$imsize
cat $0.$config.$freq.$harange.$nchan.$imsize
cat $0.$model.results

end:
